/*
Module : HTTPDOWNLOADDLG.CPP
Purpose: Defines the implementation for an MFC dialog which performs HTTP downloads
         similiar to the Internet Explorer download dialog
Created: PJN / 14-11-1999
History: PJN / 25-01-2000 1. Fixed a problem where server authentication was not being detected correctly,
                          while proxy authentication was being handled.
                          2. Updated the way and periodicity certain UI controls are updated during the 
                          HTTP download
         PJN / 12-06-2000 1. Now fully supports HTTPS downloads.
         PJN / 02-07-2000 1. Now supports resuming downloads
         PJN / 28-04-2001 1. Updated Copyright Information
                          2. You need to have the Latest Wininet header files installed to compile the code. This 
                          will not be the case with VC 5, which you will also have to install the plaform SDK also.
                          2. Now includes support for the following fields:
                             m_sProxyUserName: You can now specify the Proxy user details
                             m_sProxyPassword: THe Proxy password to use
                             m_ConnectionType: You can now specify the HTTP connection type
                             m_bPromptFileOverwrite: Should you be prompted for file overwrites
                             m_bPromptForProxyDetails: Should you be prompted for Proxy authentication
                             m_bPromptForHTTPDetails: Should you be prompted for HTTP authentication
                             m_sUserAgent: The User Agent value to use in HTTP requests
                          3. Fixed a problem in downloading files using HTTPS
                          4. Provided a OnSetOptions method to allow customisation through deriving your own
                          class from CHttpDownloadDlg
         PJN / 17-06-2001 1. Now provides Bandwidth throtling support. Thanks to Karim Mribti for
                          this good addition.
                          2. Code now uses InternetReadFile instead of InternetReadFileEx as InternetReadFileEx
                          is not implemented under Unicode.
         PJN / 10-10-2001 1. Fixed a problem where old style MFC exception handling was being used
                          instead of C++ standard.
                          2. Code is now independent of Platform SDK meaning that it can compile on VC 5 without
                          need for having the Platform SDK installed.
         PJN / 03-12-2001 1. Tidied up support for resumed downloads
         PJN / 02-02-2002 1. Fixed a bug where the local downloaded file would get appended to even when you are
                             not resuming the download. Thanks to Davide Zaccanti for spotting this problem.
                          2. Updated documentation to include explicit requirements for the class.
         PJN / 20-04-2002 1. Fixed bug in call to AfxBeginThread. Thanks to mark anderson for spotting this 
                          problem.
                          2. Fixed a redraw glitch where the progress control was being updated twice instead of
                          once each time for each percentage. Again thanks to Mark Anderson for spotting this
                          problem.
         PJN / 30-10-2002 1. Fixed a problem with seeking to the correct position when resuming a download. Thanks
                          to Liping Dai for this bug report.
         PJN / 06-01-2003 1. Fixed a bug where using HTTPS on a non standard port would cause the download to fail.
                          Thanks to "Sundar" for spotting this problem
                          2. Fixed a bug in the displaying of errors which use the strings 
                          "IDS_HTTPDOWNLOAD_INVALID_SERVER_RESPONSE" and "IDS_HTTPDOWNLOAD_INVALID_SERVER_RESPONSE"
         PJN / 19-01-2003 1. Fixed a bug in calling InternetReadFile when doing proxy authentication which could 
                          have resulted in an infinite loop occurring. Thanks to Nathan Lewis for reporting this.
                          2. Reworked the code which queries for the HTTP status code to use the HTTP_QUERY_FLAG_NUMBER
                          flag in the call to HttpQueryInfo. Again thanks to Nathan Lewis for suggesting this.
                          3. Reworked the code which queries for the HTTP content length code to use the 
                          HTTP_QUERY_FLAG_NUMBER flag in the call to HttpQueryInfo. Again thanks to Nathan Lewis 
                          for suggesting this.
         PJN / 29-06-2003 1. The progress control now uses a range of 0 - 100 percent rather than the actual file size. 
                          This prevents overflow with the SetRange method. I could use SetRange32, but then the class
                          would depend on having an updated version of the common control dll installed.
                          2. Fixed a problem in OnStatusCallBack when compiled for Unicode
         PJN / 05-02-2004 1. Updated the way the progress control is displayed when a resumed download is performed.
                          Thanks to David C. Hochrein for reporting this addition.
         PJN / 12-02-2004 1. Class now automatically pulls in wininet.lib support via a #pragma. This means that client
                          apps no longer need to update the linker settings.
         PJN / 11-11-2004 1. Fixed an issue in the code where the request is only reissued when authentication errors 
                          occur and the code needs to prompt the user. Thanks to Thomas Kiesswetter for reporting
                          this problem and provided the fix.
                          2. Addition of a CHTTPDOWNLOADDLG_EXT_CLASS preprocessor macro which allows the class
                          to be easily added to an extension dll
                          3. Fixed a bug when errors are displayed when QueryStatusCode fails.
         PJN / 05-03-2005 1. Tidied up the way the file to be downloaded is closed and deleted if the download 
                          fails. Thanks to Guillermo for reporting this bug.
                          2. Fixed a bug in the main logic of the class which caused the http or proxy credentials
                          dialog to be continually displayed even when you hit cancel. Thanks to Guillermo for 
                          reporting this bug and providing the fix.
                          3. Optimized the code which calls GetLastError.
                          4. Display of error messages now uses FormatMessage to display a more friendlier
                          value.
         PJN / 04-04-2005 1. Fix for bug when providing all information in the dialog for both proxy and user
                          validation and one uses the option to connect directly via the proxy and no prompting is checked 
                          then when the code 407 is received the code resends with proxy info and gets a 401 code. This time 
                          it reports the error as the bool flag for bRetrying is true. Thanks to Stephen Miller 
                          for providing the fix for this problem.




Copyright (c) 1999 - 2005 by PJ Naughter.  (Web: www.naughter.com, Email: pjna@naughter.com)

All rights reserved.

Copyright / Usage Details:

You are allowed to include the source code in any product (commercial, shareware, freeware or otherwise) 
when your product is released in binary form. You are allowed to modify the source code in any way you want 
except you cannot modify the copyright details at the top of each module. If you want to distribute source 
code with your application, then you are only allowed to distribute versions released by the author. This is 
to maintain a single distribution point for the source code. 

All rights reserved.

*/



/////////////////////////////////  Includes  //////////////////////////////////

#include "stdafx.h"
#include "resource.h"
#include "HttpDownloadDlg.h"
#ifndef __AFXPRIV_H__
#pragma message("To avoid this message, please put afxpriv.h in your PCH (normally stdafx.h)")
#include <afxpriv.h>
#endif



///////////////////////////////// Macros / Defines ////////////////////////////

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//Defines taken from Platform SDK's version of wininet.h to allow code to compile in VC 5 without Platform SDK being needed

#ifndef INTERNET_FLAG_PRAGMA_NOCACHE    
#define INTERNET_FLAG_PRAGMA_NOCACHE    0x00000100  // asking wininet to add "pragma: no-cache"
#endif

#ifndef INTERNET_OPTION_PROXY_USERNAME
#define INTERNET_OPTION_PROXY_USERNAME          43
#endif

#ifndef INTERNET_OPTION_PROXY_PASSWORD
#define INTERNET_OPTION_PROXY_PASSWORD          44
#endif

#ifndef HTTP_STATUS_PARTIAL_CONTENT
#define HTTP_STATUS_PARTIAL_CONTENT     206
#endif


const UINT WM_HTTPDOWNLOAD_THREAD_FINISHED = WM_APP + 1;

//Link to the wininet lib file automatically
#pragma comment(lib, "wininet.lib")



///////////////////////////////// Implementation //////////////////////////////

IMPLEMENT_DYNAMIC(CHttpDownloadDlg, CDialog);

CHttpDownloadDlg::CHttpDownloadDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CHttpDownloadDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CHttpDownloadDlg)
	//}}AFX_DATA_INIT
  m_hInternetSession = NULL;
  m_hHttpConnection = NULL;
  m_hHttpFile = NULL;
  m_bAbort = FALSE;
  m_bSafeToClose = FALSE;
  m_pThread = NULL;
  m_dwStartPos = 0;
  m_ConnectionType = UsePreConfig;
  m_bPromptFileOverwrite = TRUE;
  m_bPromptForProxyDetails = TRUE;
  m_bPromptForHTTPDetails = TRUE;
  m_dbLimit = 0;
  m_dwServiceType = 0;
}

void CHttpDownloadDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CHttpDownloadDlg)
	DDX_Control(pDX, IDC_STATUS, m_ctrlStatus);
	DDX_Control(pDX, IDC_TRANSFER_RATE, m_ctrlTransferRate);
	DDX_Control(pDX, IDC_TIMELEFT, m_ctrlTimeLeft);
	DDX_Control(pDX, IDC_PROGRESS1, m_ctrlProgress);
	DDX_Control(pDX, IDC_FILESTATUS, m_ctrlFileStatus);
	DDX_Control(pDX, IDC_ANIMATE1, m_ctrlAnimate);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CHttpDownloadDlg, CDialog)
	//{{AFX_MSG_MAP(CHttpDownloadDlg)
	ON_WM_DESTROY()
	ON_WM_CLOSE()
	//}}AFX_MSG_MAP
  ON_MESSAGE(WM_HTTPDOWNLOAD_THREAD_FINISHED, OnThreadFinished)
END_MESSAGE_MAP()

LRESULT CHttpDownloadDlg::OnThreadFinished(WPARAM wParam, LPARAM /*lParam*/)
{
  //It's now safe to close since the thread has signaled us
  m_bSafeToClose = TRUE;

  //Stop the animation
  m_ctrlAnimate.Stop();

  BOOL bDeleteDownloadFile = FALSE;

  //If an error occured display the message box
  if (m_bAbort)
  {
    bDeleteDownloadFile = TRUE;
    EndDialog(IDCANCEL);
  }
  else if (wParam)
  {
    bDeleteDownloadFile = TRUE;
    AfxMessageBox(m_sError);
    EndDialog(IDCANCEL);
  }
  else
    EndDialog(IDOK);

  if (bDeleteDownloadFile)
  {
    try
    {
      m_FileToWrite.Close();
    }
    catch(CFileException* pEx)
    {
      pEx->Delete();
    }
    ::DeleteFile(m_sFileToDownloadInto);
  }

  return 0L;
}

CString CHttpDownloadDlg::GetErrorMessage(DWORD dwError)
{
  //use the Wininet dll also to find error strings
  HMODULE hWininet = LoadLibrary(_T("WININET.DLL"));

  CString rVal;
  LPTSTR pstrError = rVal.GetBuffer(4096);

	LPTSTR lpBuffer;
	BOOL bRet = FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_FROM_HMODULE, hWininet,  dwError, MAKELANGID(LANG_NEUTRAL, SUBLANG_SYS_DEFAULT), (LPTSTR) &lpBuffer, 0, NULL);
	if (bRet == FALSE)
		*pstrError = '\0';
	else
	{
		lstrcpyn(pstrError, lpBuffer, 4096);
		LocalFree(lpBuffer);
	}
  rVal.ReleaseBuffer();

  //Don't forget to unload wininet before we return
  if (hWininet)
    FreeLibrary(hWininet);

	return rVal;
}

BOOL CHttpDownloadDlg::OnInitDialog() 
{
  //Let the parent class do its thing
	CDialog::OnInitDialog();

  //Setup the animation control
  m_ctrlAnimate.Open(IDR_HTTPDOWNLOAD_ANIMATION);

  //Validate the URL
  ASSERT(m_sURLToDownload.GetLength()); //Did you forget to specify the file to download
  if (!AfxParseURL(m_sURLToDownload, m_dwServiceType, m_sServer, m_sObject, m_nPort))
  {
    //Try sticking "http://" before it
    m_sURLToDownload = _T("http://") + m_sURLToDownload;
    if (!AfxParseURL(m_sURLToDownload, m_dwServiceType, m_sServer, m_sObject, m_nPort))
    {
      TRACE(_T("CHttpDownloadDlg::OnInitDialog, Failed to parse the URL: %s\n"), m_sURLToDownload);
      EndDialog(IDCANCEL);
      return TRUE;
    }
  }

  //Check to see if the file we are downloading to exists and if
  //it does, then ask the user if they were it overwritten
  CFileStatus fs;
  ASSERT(m_sFileToDownloadInto.GetLength());
  BOOL bDownloadFileExists = CFile::GetStatus(m_sFileToDownloadInto, fs);
  if (bDownloadFileExists && m_dwStartPos == 0 && m_bPromptFileOverwrite)
  {
    CString sMsg;
    AfxFormatString1(sMsg, IDS_HTTPDOWNLOAD_OK_TO_OVERWRITE, m_sFileToDownloadInto);
    if (AfxMessageBox(sMsg, MB_YESNO) != IDYES)
    {
      TRACE(_T("CHttpDownloadDlg::OnInitDialog, Failed to confirm file overwrite, download aborted\n"));
      EndDialog(IDCANCEL);
      return TRUE;
    }
  }

  //Try and open the file we will download into
  DWORD dwFileFlags = 0;
  if (bDownloadFileExists && (m_dwStartPos > 0))
    dwFileFlags = CFile::modeCreate | CFile::modeNoTruncate | CFile::modeWrite | CFile::shareDenyWrite;
  else
    dwFileFlags = CFile::modeCreate | CFile::modeWrite | CFile::shareDenyWrite;
  if (!m_FileToWrite.Open(m_sFileToDownloadInto, dwFileFlags))
  {
    DWORD dwError = GetLastError();
    TRACE(_T("CHttpDownloadDlg::OnInitDialog, Failed to open the file to download into, Error:%d\n"), dwError);
    CString sMsg;
    AfxFormatString1(sMsg, IDS_HTTPDOWNLOAD_FAIL_FILE_OPEN, GetErrorMessage(dwError));
    AfxMessageBox(sMsg);
    EndDialog(IDCANCEL);
    return TRUE;
  }
  else
  {
		//Seek to the end of the file
    try
	  {
      m_FileToWrite.Seek(m_dwStartPos, CFile::begin); 
      m_FileToWrite.SetLength(m_dwStartPos);
	  }
    catch(CFileException* pEx)                                         
    {
      TRACE(_T("CHttpDownloadDlg::OnInitDialog, An exception occured while set file pointer position, Error:%d\n"), pEx->m_lOsError);
      CString sMsg;
      AfxFormatString1(sMsg, IDS_HTTPDOWNLOAD_FAIL_FILE_SEEK, GetErrorMessage(pEx->m_lOsError));
		  pEx->Delete();
      AfxMessageBox(sMsg);
		  EndDialog(IDCANCEL);
      return TRUE;
    }	
  }

  //Pull out just the filename component
  int nSlash = m_sObject.ReverseFind(_T('/'));
  if (nSlash == -1)
    nSlash = m_sObject.ReverseFind(_T('\\'));
  if (nSlash != -1 && m_sObject.GetLength() > 1)
    m_sFilename = m_sObject.Right(m_sObject.GetLength() - nSlash - 1);
  else
    m_sFilename = m_sObject;

	//Set the file status text
  CString sFileStatus;
  ASSERT(m_sObject.GetLength());
  ASSERT(m_sServer.GetLength());
  AfxFormatString2(sFileStatus, IDS_HTTPDOWNLOAD_FILESTATUS, m_sFilename, m_sServer);
  m_ctrlFileStatus.SetWindowText(sFileStatus);

  //Spin off the background thread which will do the actual downloading
  m_pThread = AfxBeginThread(_DownloadThread, this, THREAD_PRIORITY_NORMAL, 0, CREATE_SUSPENDED);
  if (m_pThread == NULL)
  {
    TRACE(_T("CHttpDownloadDlg::OnInitDialog, Failed to create download thread, dialog is aborting\n"));
    EndDialog(IDCANCEL);
    return TRUE;
  }
  m_pThread->m_bAutoDelete = FALSE;
  m_pThread->ResumeThread();
	
	return TRUE;
}

UINT CHttpDownloadDlg::_DownloadThread(LPVOID pParam)
{
  //Convert from the SDK world to the C++ world
  CHttpDownloadDlg* pDlg = (CHttpDownloadDlg*) pParam;
  ASSERT(pDlg);
  ASSERT(pDlg->IsKindOf(RUNTIME_CLASS(CHttpDownloadDlg)));
  pDlg->DownloadThread();
  return 0;
}

void CHttpDownloadDlg::SetPercentage(int nPercentage)
{
  //Change the progress control
  m_ctrlProgress.SetPos(nPercentage);

  //Change the caption text
  CString sPercentage;
  sPercentage.Format(_T("%d"), nPercentage);
  CString sCaption;
  AfxFormatString2(sCaption, IDS_HTTPDOWNLOAD_PERCENTAGE, sPercentage, m_sFilename);
  SetWindowText(sCaption);
}

void CHttpDownloadDlg::SetTimeLeft(DWORD dwSecondsLeft, DWORD dwBytesRead, DWORD dwFileSize)
{
  CString sCopied;
  if (dwBytesRead < 1024)
  {
    CString sBytes;
    sBytes.Format(_T("%d"), dwBytesRead);
    AfxFormatString1(sCopied, IDS_HTTPDOWNLOAD_BYTES, sBytes);
  }
  else if (dwBytesRead < 1048576)
  {
    CString sKiloBytes;
    sKiloBytes.Format(_T("%0.1f"), dwBytesRead/1024.0);
    AfxFormatString1(sCopied, IDS_HTTPDOWNLOAD_KILOBYTES, sKiloBytes);
  }
  else
  {
    CString sMegaBytes;
    sMegaBytes.Format(_T("%0.2f"), dwBytesRead/1048576.0);
    AfxFormatString1(sCopied, IDS_HTTPDOWNLOAD_MEGABYTES, sMegaBytes);
  }

  CString sTotal;
  if (dwFileSize < 1024)
  {
    CString sBytes;
    sBytes.Format(_T("%d"), dwFileSize);
    AfxFormatString1(sTotal, IDS_HTTPDOWNLOAD_BYTES, sBytes);
  }
  else if (dwFileSize < 1048576)
  {
    CString sKiloBytes;
    sKiloBytes.Format(_T("%0.1f"), dwFileSize/1024.0);
    AfxFormatString1(sTotal, IDS_HTTPDOWNLOAD_KILOBYTES, sKiloBytes);
  }
  else
  {
    CString sMegaBytes;
    sMegaBytes.Format(_T("%0.2f"), dwFileSize/1048576.0);
    AfxFormatString1(sTotal, IDS_HTTPDOWNLOAD_MEGABYTES, sMegaBytes);
  }

  CString sOf;
  AfxFormatString2(sOf, IDS_HTTPDOWNLOAD_OF, sCopied, sTotal);

  CString sTime;
  if (dwSecondsLeft < 60)
  {
    CString sSeconds;
    sSeconds.Format(_T("%d"), dwSecondsLeft);
    AfxFormatString1(sTime, IDS_HTTPDOWNLOAD_SECONDS, sSeconds);
  }
  else
  {
    DWORD dwMinutes = dwSecondsLeft / 60;
    DWORD dwSeconds = dwSecondsLeft % 60;
    CString sSeconds;
    sSeconds.Format(_T("%d"), dwSeconds);
    CString sMinutes;
    sMinutes.Format(_T("%d"), dwMinutes);
    if (dwSeconds == 0)
      AfxFormatString1(sTime, IDS_HTTPDOWNLOAD_MINUTES, sMinutes);
    else
      AfxFormatString2(sTime, IDS_HTTPDOWNLOAD_MINUTES_AND_SECONDS, sMinutes, sSeconds);
  }

  CString sTimeLeft;
  AfxFormatString2(sTimeLeft, IDS_HTTPDOWNLOAD_TIMELEFT, sTime, sOf);
  m_ctrlTimeLeft.SetWindowText(sTimeLeft);
}

void CHttpDownloadDlg::SetStatus(const CString& sCaption)
{
  m_ctrlStatus.SetWindowText(sCaption);
}

void CHttpDownloadDlg::SetStatus(UINT nID)
{
  CString sCaption;
  sCaption.LoadString(nID);
  SetStatus(sCaption);
}

void CHttpDownloadDlg::SetStatus(UINT nID, const CString& lpsz1)
{
  CString sStatus;
  AfxFormatString1(sStatus, nID, lpsz1);
  SetStatus(sStatus);
}

void CHttpDownloadDlg::SetTransferRate(double KbPerSecond)
{
  CString sRate;
  if (KbPerSecond < 1)
  {
    CString sBytesPerSecond;
    sBytesPerSecond.Format(_T("%0.0f"), KbPerSecond*1024);
    AfxFormatString1(sRate, IDS_HTTPDOWNLOAD_BYTESPERSECOND, sBytesPerSecond);
  }
  else if (KbPerSecond < 10)
  {
    CString sKiloBytesPerSecond;
    sKiloBytesPerSecond.Format(_T("%0.2f"), KbPerSecond);
    AfxFormatString1(sRate, IDS_HTTPDOWNLOAD_KILOBYTESPERSECOND, sKiloBytesPerSecond);
  }
  else
  {
    CString sKiloBytesPerSecond;
    sKiloBytesPerSecond.Format(_T("%0.0f"), KbPerSecond);
    AfxFormatString1(sRate, IDS_HTTPDOWNLOAD_KILOBYTESPERSECOND, sKiloBytesPerSecond);
  }
  m_ctrlTransferRate.SetWindowText(sRate);
}

void CHttpDownloadDlg::PlayAnimation()
{
  m_ctrlAnimate.Play(0, (UINT)-1, (UINT)-1);
}

void CHttpDownloadDlg::HandleThreadErrorWithLastError(UINT nIDError, DWORD dwLastError)
{
  //Form the error string to report
  CString sError;
  if (dwLastError)
    sError.Format(_T("%s"), GetErrorMessage(dwLastError));
  else
    sError.Format(_T("%s"), GetErrorMessage(GetLastError()));
  AfxFormatString1(m_sError, nIDError, sError);

  PostMessage(WM_HTTPDOWNLOAD_THREAD_FINISHED, 1);
}

void CHttpDownloadDlg::HandleThreadErrorWithStatusCode(UINT nIDError, DWORD dwStatusCode)
{
  //Form the error string to report
  CString sError;
  sError.Format(_T("%d"), dwStatusCode);
  AfxFormatString1(m_sError, nIDError, sError);

  PostMessage(WM_HTTPDOWNLOAD_THREAD_FINISHED, 1);
}

void CHttpDownloadDlg::HandleThreadError(UINT nIDError)
{
  //Form the error string to report
  m_sError.LoadString(nIDError);

  PostMessage(WM_HTTPDOWNLOAD_THREAD_FINISHED, 1);
}

BOOL CHttpDownloadDlg::OnSetOptions()
{
  //Do nothing but your derviced class could override this function and
  //do things ssuch as InternetSetOptions(m_hInternetSession, INTERNET_OPTION_CONNECT_RETRIES, ..) etc.

  return TRUE; //To continue processing
}

BOOL CHttpDownloadDlg::QueryStatusNumber(HINTERNET hInternet, DWORD dwFlag, DWORD& dwCode)
{
  dwCode = 0;
  DWORD dwSize = sizeof(DWORD);
  return HttpQueryInfo(hInternet, dwFlag | HTTP_QUERY_FLAG_NUMBER, &dwCode, &dwSize, NULL);
}

BOOL CHttpDownloadDlg::QueryStatusCode(HINTERNET hInternet, DWORD& dwCode) 
{
  return QueryStatusNumber(hInternet, HTTP_QUERY_STATUS_CODE, dwCode);
}

BOOL CHttpDownloadDlg::QueryContentLength(HINTERNET hInternet, DWORD& dwCode) 
{
  return QueryStatusNumber(hInternet, HTTP_QUERY_CONTENT_LENGTH, dwCode);
}

void CHttpDownloadDlg::DownloadThread()
{
  //Create the Internet session handle
  ASSERT(m_hInternetSession == NULL);

  switch (m_ConnectionType)
  {
    case UsePreConfig:
    {
      m_hInternetSession = ::InternetOpen(m_sUserAgent.GetLength() ? m_sUserAgent : AfxGetAppName(), INTERNET_OPEN_TYPE_PRECONFIG, NULL, NULL, 0);    
      break;
    }
    case DirectToInternet:
    {
      m_hInternetSession = ::InternetOpen(m_sUserAgent.GetLength() ? m_sUserAgent : AfxGetAppName(), INTERNET_OPEN_TYPE_DIRECT, NULL, NULL, 0);    
      break;
    }
    case UseProxy:
    {
      ASSERT(m_sProxyServer.GetLength()); //You need to give me a proxy Server
      m_hInternetSession = ::InternetOpen(m_sUserAgent.GetLength() ? m_sUserAgent : AfxGetAppName(), INTERNET_OPEN_TYPE_PROXY, m_sProxyServer, NULL, 0);    
      break;
    }
    default:  
    {
      ASSERT(FALSE);
      break;
    }
  }

  if (m_hInternetSession == NULL)
  {
    DWORD dwError = GetLastError();
    TRACE(_T("CHttpDownloadDlg::DownloadThread, Failed in call to InternetOpen, Error:%d\n"), dwError);
    HandleThreadErrorWithLastError(IDS_HTTPDOWNLOAD_GENERIC_ERROR, dwError);
    return;
  }

  //Should we exit the thread
  if (m_bAbort)
  {
    PostMessage(WM_HTTPDOWNLOAD_THREAD_FINISHED);
    return;
  }  

  //Setup the status callback function
  if (::InternetSetStatusCallback(m_hInternetSession, _OnStatusCallBack) == INTERNET_INVALID_STATUS_CALLBACK)
  {
    DWORD dwError = GetLastError();
    TRACE(_T("CHttpDownloadDlg::DownloadThread, Failed in call to InternetSetStatusCallback, Error:%d\n"), dwError);
    HandleThreadErrorWithLastError(IDS_HTTPDOWNLOAD_GENERIC_ERROR, dwError);
    return;
  }

  //Should we exit the thread
  if (m_bAbort)
  {
    PostMessage(WM_HTTPDOWNLOAD_THREAD_FINISHED);
    return;
  }  

  //Make the connection to the HTTP server          
  ASSERT(m_hHttpConnection == NULL);
  if (m_sHTTPUserName.GetLength())
    m_hHttpConnection = ::InternetConnect(m_hInternetSession, m_sServer, m_nPort, m_sHTTPUserName, 
                                          m_sHTTPPassword, INTERNET_SERVICE_HTTP, 0, (DWORD) this);
  else
    m_hHttpConnection = ::InternetConnect(m_hInternetSession, m_sServer, m_nPort, NULL, 
                                          NULL, INTERNET_SERVICE_HTTP, 0, (DWORD) this);
  if (m_hHttpConnection == NULL)
  {
    DWORD dwError = GetLastError();
    TRACE(_T("CHttpDownloadDlg::DownloadThread, Failed in call to InternetConnect, Error:%d\n"), dwError);
    HandleThreadErrorWithLastError(IDS_HTTPDOWNLOAD_FAIL_CONNECT_SERVER, dwError);
    return;
  }

  //Should we exit the thread
  if (m_bAbort)
  {
    PostMessage(WM_HTTPDOWNLOAD_THREAD_FINISHED);
    return;
  }  

  //Call the virtual function to allow session customisation
  if (!OnSetOptions())
  {
    TRACE(_T("CHttpDownloadDlg::DownloadThread, Failed in call to OnSetOptions\n"));
    return;
  }

  //Start the animation to signify that the download is taking place
  PlayAnimation();

  //Issue the request to read the file
  LPCTSTR ppszAcceptTypes[2];
  ppszAcceptTypes[0] = _T("*/*");  //We support accepting any mime file type since this is a simple download of a file
  ppszAcceptTypes[1] = NULL;
  ASSERT(m_hHttpFile == NULL);
 	DWORD dwFlags = INTERNET_FLAG_RELOAD | INTERNET_FLAG_DONT_CACHE | INTERNET_FLAG_PRAGMA_NOCACHE | INTERNET_FLAG_KEEP_CONNECTION;
	if (m_dwServiceType == AFX_INET_SERVICE_HTTPS) 
		dwFlags	|= (INTERNET_FLAG_SECURE | INTERNET_FLAG_IGNORE_CERT_CN_INVALID | INTERNET_FLAG_IGNORE_CERT_DATE_INVALID);
  m_hHttpFile = HttpOpenRequest(m_hHttpConnection, NULL, m_sObject, _T("HTTP/1.1"), NULL, ppszAcceptTypes, dwFlags, (DWORD) this);
  if (m_hHttpFile == NULL)
  {
    DWORD dwError = GetLastError();
    TRACE(_T("CHttpDownloadDlg::DownloadThread, Failed in call to HttpOpenRequest, Error:%d\n"), dwError);
    HandleThreadErrorWithLastError(IDS_HTTPDOWNLOAD_FAIL_CONNECT_SERVER, dwError);
    return;
  }

  //Should we exit the thread.
  //The purpose is to check if user has pressed the cancel button
  if (m_bAbort)
  {
    PostMessage(WM_HTTPDOWNLOAD_THREAD_FINISHED);
    return;
  }  

  BOOL bRetryingProxy = FALSE;
  BOOL bRetryingUser = FALSE;
//label used to jump to if we need to resend the request
resend:

  //Issue the request
  CString sRange;
  if (m_dwStartPos != 0)  //we will build the range request.
	  sRange.Format(_T("Range: bytes=%d-\r\n"), m_dwStartPos);
  BOOL bSend = FALSE;
  if (sRange.IsEmpty())
    bSend = ::HttpSendRequest(m_hHttpFile, NULL, 0, NULL, 0);
  else
    bSend = ::HttpSendRequest(m_hHttpFile, sRange, sRange.GetLength(), NULL, 0);
  if (!bSend)
  {
    DWORD dwError = GetLastError();
    TRACE(_T("CHttpDownloadDlg::DownloadThread, Failed in call to HttpSendRequest, Error:%d\n"), dwError);
    HandleThreadErrorWithLastError(IDS_HTTPDOWNLOAD_FAIL_CONNECT_SERVER, dwError);
    return;
  }

  //Handle the status code in the response
  BOOL bPrompt = FALSE;
  DWORD dwStatusCode = 0;
	if (!QueryStatusCode(m_hHttpFile, dwStatusCode))
  {
    DWORD dwError = ::GetLastError();
    TRACE(_T("CHttpDownloadDlg::DownloadThread, Failed in call to HttpQueryInfo for HTTP query status code, Error:%d\n"), dwError);
    HandleThreadErrorWithLastError(IDS_HTTPDOWNLOAD_INVALID_SERVER_RESPONSE, dwError);
    return;
  }

  //discard any outstanding data if required
  if (dwStatusCode == HTTP_STATUS_PROXY_AUTH_REQ || dwStatusCode == HTTP_STATUS_DENIED)
  {
    char szData[1024];
    DWORD dwSize = 0;
    do 
    {
      ::InternetReadFile(m_hHttpFile, szData, sizeof(szData), &dwSize);
    } 
    while (dwSize != 0);
  }

  if (dwStatusCode == HTTP_STATUS_PROXY_AUTH_REQ) //Proxy authentication required
  {
    int nProxyUserLength = m_sProxyUserName.GetLength();
    if (nProxyUserLength && !bRetryingProxy)
    {
      InternetSetOption(m_hHttpFile, INTERNET_OPTION_PROXY_USERNAME, (LPVOID) m_sProxyUserName.operator LPCTSTR(), (nProxyUserLength+1) * sizeof(TCHAR));
      InternetSetOption(m_hHttpFile, INTERNET_OPTION_PROXY_PASSWORD, (LPVOID) m_sProxyPassword.operator LPCTSTR(), (m_sProxyPassword.GetLength()+1) * sizeof(TCHAR));
      bRetryingProxy = TRUE;
      goto resend;
    }
    else if (m_bPromptForProxyDetails)
      bPrompt = TRUE;
    else
    {
      TRACE(_T("CHttpDownloadDlg::DownloadThread, Failed to retrieve a HTTP OK or partial content status, Status Code:%d\n"), dwStatusCode);
      HandleThreadErrorWithStatusCode(IDS_HTTPDOWNLOAD_INVALID_HTTP_RESPONSE, dwStatusCode);
      return;
    }
  }
  else if (dwStatusCode == HTTP_STATUS_DENIED) //HTTP authentication required
  {
    int nHTTPUserLength = m_sHTTPUserName.GetLength();
    if (nHTTPUserLength && !bRetryingUser)
    {
      InternetSetOption(m_hHttpFile, INTERNET_OPTION_USERNAME, (LPVOID) m_sHTTPUserName.operator LPCTSTR(), (nHTTPUserLength+1) * sizeof(TCHAR));
      InternetSetOption(m_hHttpFile, INTERNET_OPTION_PASSWORD, (LPVOID) m_sHTTPPassword.operator LPCTSTR(), (m_sHTTPPassword.GetLength()+1) * sizeof(TCHAR));
      bRetryingUser = TRUE;
      goto resend;
    }
    else if (m_bPromptForHTTPDetails)
      bPrompt = TRUE;
    else
    {
      TRACE(_T("CHttpDownloadDlg::DownloadThread, Failed to retrieve a HTTP OK or partial content status, Status Code:%d\n"), dwStatusCode);
      HandleThreadErrorWithStatusCode(IDS_HTTPDOWNLOAD_INVALID_HTTP_RESPONSE, dwStatusCode);
      return;
    }
  }
  else if (dwStatusCode != HTTP_STATUS_OK && dwStatusCode != HTTP_STATUS_PARTIAL_CONTENT)
  {
    TRACE(_T("CHttpDownloadDlg::DownloadThread, Failed to retrieve a HTTP OK or partial content status, Status Code:%d\n"), dwStatusCode);
    HandleThreadErrorWithStatusCode(IDS_HTTPDOWNLOAD_INVALID_HTTP_RESPONSE, dwStatusCode);
    return;
  }

  if (bPrompt)
  {
    //Bring up the standard authentication dialog if required
    DWORD dwError = ::InternetErrorDlg(GetSafeHwnd(), m_hHttpFile, ERROR_INTERNET_INCORRECT_PASSWORD, 
                                       FLAGS_ERROR_UI_FILTER_FOR_ERRORS | FLAGS_ERROR_UI_FLAGS_GENERATE_DATA | FLAGS_ERROR_UI_FLAGS_CHANGE_OPTIONS, NULL);
    if (dwError == ERROR_INTERNET_FORCE_RETRY)
      goto resend;
    else if (dwError != ERROR_SUCCESS)
    {
      TRACE(_T("CHttpDownloadDlg::DownloadThread, HTTP authentication canceled by user, Error Code:%d\n"), dwError);
      HandleThreadErrorWithLastError(IDS_HTTPDOWNLOAD_ABORTING_TRANSFER, dwError);
      return;
    }
    else
    {
      m_bAbort = TRUE;
      PostMessage(WM_HTTPDOWNLOAD_THREAD_FINISHED);
      return;
    }
  }

  //Update the status control to reflect that we are getting the file information
  SetStatus(IDS_HTTPDOWNLOAD_GETTING_FILE_INFORMATION);

  // Get the length of the file.            
  DWORD dwFileSize = 0;
  BOOL bGotFileSize = FALSE;
  if (QueryContentLength(m_hHttpFile, dwFileSize))
  {
    //Set the progress control range
    bGotFileSize = TRUE;
    m_ctrlProgress.SetRange(0, 100);
  }

  //Update the status to say that we are now downloading the file
  SetStatus(IDS_HTTPDOWNLOAD_RETREIVEING_FILE);

  //Now do the actual read of the file
  DWORD dwStartTicks = ::GetTickCount();
  DWORD dwCurrentTicks = dwStartTicks;
  DWORD dwBytesRead = 0;
  char szReadBuf[1024];
  DWORD dwTotalBytesRead = 0;
  DWORD dwLastTotalBytes = 0;
  DWORD dwLastPercentage = 0;

  do
  {
    if (!::InternetReadFile(m_hHttpFile, szReadBuf, 1024, &dwBytesRead))
    {
      DWORD dwError = GetLastError();
      TRACE(_T("CHttpDownloadDlg::DownloadThread, Failed in call to InternetReadFile, Error:%d\n"), dwError);
      HandleThreadErrorWithLastError(IDS_HTTPDOWNLOAD_ERROR_READFILE, dwError);
      return;
    }
    else
    {
      if (dwBytesRead && !m_bAbort)
      {
        //Write the data to file
        try
        {
          m_FileToWrite.Write(szReadBuf, dwBytesRead);
        }
        catch(CFileException* pEx)
        {
          TRACE(_T("CHttpDownloadDlg::DownloadThread, An exception occured while writing to the download file\n"));
          HandleThreadErrorWithLastError(IDS_HTTPDOWNLOAD_ERROR_READFILE, pEx->m_lOsError);
          pEx->Delete();
          return;
        }

        // For bandwidth throttling
		    if (m_dbLimit > 0.0f) 
        {
		      double t = (double)(GetTickCount() - dwStartTicks);
		      double q = (double)((double)dwTotalBytesRead / t);
		     
		      if (q > m_dbLimit)	 
  			    Sleep((DWORD)((((q*t)/m_dbLimit)-t)));
		    }
      }

      //Increment the total number of bytes read
      dwTotalBytesRead += dwBytesRead;  

      //Update the UI
      UpdateControlsDuringTransfer(dwStartTicks, dwCurrentTicks, dwTotalBytesRead, dwLastTotalBytes, 
                                   dwLastPercentage, bGotFileSize, dwFileSize);
    }
  } 
  while (dwBytesRead && !m_bAbort);

  //Just close the file before we return
  m_FileToWrite.Close();

  //We're finished
  PostMessage(WM_HTTPDOWNLOAD_THREAD_FINISHED);
}

void CHttpDownloadDlg::UpdateControlsDuringTransfer(DWORD dwStartTicks, DWORD& dwCurrentTicks, DWORD dwTotalBytesRead, DWORD& dwLastTotalBytes, 
                                                    DWORD& dwLastPercentage, BOOL bGotFileSize, DWORD dwFileSize)
{
  if (bGotFileSize)
  {
    //Update the percentage downloaded in the caption
    DWORD dwPercentage = (DWORD) ((dwTotalBytesRead + m_dwStartPos) * 100.0 / (dwFileSize + m_dwStartPos));
    if (dwPercentage != dwLastPercentage)
    {
      //Update the progress control bar
      SetPercentage(dwPercentage);
      dwLastPercentage = dwPercentage;
    }
  }

  //Update the transfer rate amd estimated time left every second
  DWORD dwNowTicks = GetTickCount();
  DWORD dwTimeTaken = dwNowTicks - dwCurrentTicks;
  if (dwTimeTaken > 1000)
  {
    double KbPerSecond = ((double)(dwTotalBytesRead) - (double)(dwLastTotalBytes)) / ((double)(dwTimeTaken));
    SetTransferRate(KbPerSecond);

    //Setup for the next time around the loop
    dwCurrentTicks = dwNowTicks;
    dwLastTotalBytes = dwTotalBytesRead;

    if (bGotFileSize)
    {
      //Update the estimated time left
      if (dwTotalBytesRead)
      {
        DWORD dwSecondsLeft = (DWORD) (((double)dwNowTicks - dwStartTicks) / dwTotalBytesRead * 
                                       (dwFileSize - dwTotalBytesRead) / 1000);
        SetTimeLeft(dwSecondsLeft, dwTotalBytesRead + m_dwStartPos, dwFileSize + m_dwStartPos);
      }
    }
  }
}

void CALLBACK CHttpDownloadDlg::_OnStatusCallBack(HINTERNET hInternet, DWORD dwContext, DWORD dwInternetStatus, 
                                                  LPVOID lpvStatusInformation, DWORD dwStatusInformationLength)
{
  //Convert from the SDK C world to the C++ world
  CHttpDownloadDlg* pDlg = (CHttpDownloadDlg*) dwContext;
  ASSERT(pDlg);
  ASSERT(pDlg->IsKindOf(RUNTIME_CLASS(CHttpDownloadDlg)));
  pDlg->OnStatusCallBack(hInternet, dwInternetStatus, lpvStatusInformation, dwStatusInformationLength);
}

void CHttpDownloadDlg::OnStatusCallBack(HINTERNET /*hInternet*/, DWORD dwInternetStatus, 
                                         LPVOID lpvStatusInformation, DWORD /*dwStatusInformationLength*/)
{
  USES_CONVERSION;

  switch (dwInternetStatus)
  {
    case INTERNET_STATUS_RESOLVING_NAME:
    {
      SetStatus(IDS_HTTPDOWNLOAD_RESOLVING_NAME, A2T((LPSTR) lpvStatusInformation));
      break;
    }
    case INTERNET_STATUS_NAME_RESOLVED:
    {
      SetStatus(IDS_HTTPDOWNLOAD_RESOLVED_NAME, A2T((LPSTR) lpvStatusInformation));
      break;
    }
    case INTERNET_STATUS_CONNECTING_TO_SERVER:
    {
      SetStatus(IDS_HTTPDOWNLOAD_CONNECTING, A2T((LPSTR) lpvStatusInformation));
      break;
    }
    case INTERNET_STATUS_CONNECTED_TO_SERVER:
    {
      SetStatus(IDS_HTTPDOWNLOAD_CONNECTED, A2T((LPSTR) lpvStatusInformation));
      break;
    }
    case INTERNET_STATUS_REDIRECT:
    {
      SetStatus(IDS_HTTPDOWNLOAD_REDIRECTING, A2T((LPSTR) lpvStatusInformation));
      break;
    }
    default:
    {
      break;
    }
  }
}

void CHttpDownloadDlg::OnDestroy() 
{
  //Wait for the worker thread to exit
  if (m_pThread)
  {
    WaitForSingleObject(m_pThread->m_hThread, INFINITE);
    delete m_pThread;
    m_pThread = NULL;
  }

  //Free up the internet handles we may be using
  if (m_hHttpFile)
  {
    ::InternetCloseHandle(m_hHttpFile);
    m_hHttpFile = NULL;
  }
  if (m_hHttpConnection)
  {
    ::InternetCloseHandle(m_hHttpConnection);
    m_hHttpConnection = NULL;
  }
  if (m_hInternetSession)
  {
    ::InternetCloseHandle(m_hInternetSession);
    m_hInternetSession = NULL;
  }

  //Let the parent class do its thing
	CDialog::OnDestroy();
}

void CHttpDownloadDlg::OnCancel() 
{
  //Just set the abort flag to TRUE and
  //disable the cancel button
  m_bAbort = TRUE;	
  GetDlgItem(IDCANCEL)->EnableWindow(FALSE);
  SetStatus(IDS_HTTPDOWNLOAD_ABORTING_TRANSFER);
}

void CHttpDownloadDlg::OnClose() 
{
  if (m_bSafeToClose)	
	  CDialog::OnClose();
  else
  {
    //Just set the abort flag to TRUE and
    //disable the cancel button
    m_bAbort = TRUE;	
    GetDlgItem(IDCANCEL)->EnableWindow(FALSE);
    SetStatus(IDS_HTTPDOWNLOAD_ABORTING_TRANSFER);
  }
}
