/*
Module : HTTPDOWNLOADDLG.H
Purpose: Defines the interface for an MFC dialog which performs HTTP downloads
         similiar to the Internet Explorer download dialog

Copyright (c) 1999 - 2005 by PJ Naughter.  (Web: www.naughter.com, Email: pjna@naughter.com)

All rights reserved.

Copyright / Usage Details:

You are allowed to include the source code in any product (commercial, shareware, freeware or otherwise) 
when your product is released in binary form. You are allowed to modify the source code in any way you want 
except you cannot modify the copyright details at the top of each module. If you want to distribute source 
code with your application, then you are only allowed to distribute versions released by the author. This is 
to maintain a single distribution point for the source code. 

*/


////////////////////////////////// Macros ///////////////////////////

#ifndef __HTTPDOWNLOADDLG_H__
#define __HTTPDOWNLOADDLG_H__

#ifndef CHTTPDOWNLOADDLG_EXT_CLASS
#define CHTTPDOWNLOADDLG_EXT_CLASS
#endif



/////////////////////////// Classes /////////////////////////////////

class CHTTPDOWNLOADDLG_EXT_CLASS CHttpDownloadDlg : public CDialog
{
public:
//Constructors / Destructors
	CHttpDownloadDlg(CWnd* pParent = NULL);

//Enums
enum ConnectionType
{
  UsePreConfig,  
  DirectToInternet,
  UseProxy,
};

//Public Member variables
  CString        m_sURLToDownload;
  CString        m_sFileToDownloadInto;
  CString        m_sProxyServer;
  CString        m_sProxyUserName;
  CString        m_sProxyPassword;
  CString        m_sHTTPUserName;
  CString        m_sHTTPPassword;
  CString        m_sUserAgent;
  ConnectionType m_ConnectionType;
  BOOL           m_bPromptFileOverwrite;
  BOOL           m_bPromptForProxyDetails;
  BOOL           m_bPromptForHTTPDetails;
  double		     m_dbLimit;          //For BANDWIDTH Throptling, The value in Bytes / Second to limit the connection to
  DWORD          m_dwStartPos;       //Offset to resume the download at   

protected:
	//{{AFX_DATA(CHttpDownloadDlg)
	enum { IDD = IDD_HTTPDOWNLOAD };
	CStatic	m_ctrlStatus;
	CStatic	m_ctrlTransferRate;
	CStatic	m_ctrlTimeLeft;
	CProgressCtrl	m_ctrlProgress;
	CStatic	m_ctrlFileStatus;
	CAnimateCtrl m_ctrlAnimate;
	//}}AFX_DATA

	//{{AFX_VIRTUAL(CHttpDownloadDlg)
	virtual void DoDataExchange(CDataExchange* pDX);
	//}}AFX_VIRTUAL

	//{{AFX_MSG(CHttpDownloadDlg)
	virtual BOOL OnInitDialog();
	afx_msg void OnDestroy();
	virtual void OnCancel();
	afx_msg void OnClose();
	//}}AFX_MSG
  afx_msg LRESULT OnThreadFinished(WPARAM wParam, LPARAM lParam);

	DECLARE_MESSAGE_MAP()
  DECLARE_DYNAMIC(CHttpDownloadDlg);

//Methods
  static void CALLBACK _OnStatusCallBack(HINTERNET hInternet, DWORD dwContext, DWORD dwInternetStatus, 
                                        LPVOID lpvStatusInformation, DWORD dwStatusInformationLength);
  static BOOL QueryStatusNumber(HINTERNET hInternet, DWORD dwFlag, DWORD& dwCode);
  static BOOL QueryStatusCode(HINTERNET hInternet, DWORD& dwCode);
  static BOOL QueryContentLength(HINTERNET hInternet, DWORD& dwCode);
  void OnStatusCallBack(HINTERNET hInternet, DWORD dwInternetStatus, 
                        LPVOID lpvStatusInformation, DWORD dwStatusInformationLength);
  static UINT _DownloadThread(LPVOID pParam);
  static CString GetErrorMessage(DWORD dwError);
  virtual BOOL OnSetOptions();
  virtual void HandleThreadErrorWithLastError(UINT nIDError, DWORD dwLastError=0);
  virtual void HandleThreadErrorWithStatusCode(UINT nIDError, DWORD dwStatusCode);
  virtual void HandleThreadError(UINT nIDError);
  virtual void DownloadThread();
  virtual void SetPercentage(int nPercentage);
  virtual void SetTimeLeft(DWORD dwSecondsLeft, DWORD dwBytesRead, DWORD dwFileSize);
  virtual void SetStatus(const CString& sCaption);
  virtual void SetStatus(UINT nID);
  virtual void SetStatus(UINT nID, const CString& lpsz1);
  virtual void SetTransferRate(double KbPerSecond);
  virtual void PlayAnimation();
  virtual void UpdateControlsDuringTransfer(DWORD dwStartTicks, DWORD& dwCurrentTicks, DWORD dwTotalBytesRead, DWORD& dwLastTotalBytes, 
                                            DWORD& dwLastPercentage, BOOL bGotFileSize, DWORD dwFileSize);

//Member variables
  CString       m_sError;
  CString       m_sServer; 
  DWORD         m_dwServiceType;
  CString       m_sObject; 
  CString       m_sFilename;
  INTERNET_PORT m_nPort;
  HINTERNET     m_hInternetSession;
  HINTERNET     m_hHttpConnection;
  HINTERNET     m_hHttpFile;
  BOOL          m_bAbort;
  BOOL          m_bSafeToClose;
  CFile         m_FileToWrite;
  CWinThread*   m_pThread;
};

#endif //__HTTPDOWNLOADDLG_H__
