#include "stdafx.h"
#include "LottoMan.h"
#include "LottoManDoc.h"
#include "MemberUI.h"
#include "SettingsUI.h"
#include "ResultsDlg.h"
#include "LottoManView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//The item which will be stored in the ItemData
struct LottoViewItem
{
  void* pData;
	int   nData;
};

IMPLEMENT_DYNCREATE(CLottoManView, CListView)

BEGIN_MESSAGE_MAP(CLottoManView, CListView)
	//{{AFX_MSG_MAP(CLottoManView)
	ON_COMMAND(ID_EDIT_SELECTALL, OnEditSelectall)
	ON_COMMAND(ID_EDIT_INVERTSELECTION, OnEditInvertselection)
	ON_COMMAND(ID_FILE_DELETE, OnFileDelete)
	ON_UPDATE_COMMAND_UI(ID_FILE_DELETE, OnUpdateFileDelete)
	ON_NOTIFY_REFLECT(NM_RETURN, OnReturn)
	ON_NOTIFY_REFLECT(NM_DBLCLK, OnDblclk)
	ON_NOTIFY_REFLECT(LVN_BEGINLABELEDIT, OnBeginlabeledit)
	ON_NOTIFY_REFLECT(LVN_ENDLABELEDIT, OnEndlabeledit)
	ON_NOTIFY_REFLECT(NM_RCLICK, OnRclick)
	ON_COMMAND(ID_FILE_OPEN_ITEM, OnFileOpenItem)
	ON_UPDATE_COMMAND_UI(ID_FILE_OPEN_ITEM, OnUpdateFileOpenItem)
	ON_COMMAND(ID_FILE_RENAME, OnFileRename)
	ON_UPDATE_COMMAND_UI(ID_FILE_RENAME, OnUpdateFileRename)
	ON_COMMAND(ID_FILE_PROPERTIES, OnFileProperties)
	ON_UPDATE_COMMAND_UI(ID_FILE_PROPERTIES, OnUpdateFileProperties)
	ON_COMMAND(ID_FILE_CHECK_RESULTS, OnFileCheckResults)
	ON_UPDATE_COMMAND_UI(ID_FILE_CHECK_RESULTS, OnUpdateFileCheckResults)
	ON_COMMAND(ID_VIEW_DETAILS, OnViewDetails)
	ON_UPDATE_COMMAND_UI(ID_VIEW_DETAILS, OnUpdateViewDetails)
	ON_UPDATE_COMMAND_UI(ID_VIEW_LARGEICON, OnUpdateViewLargeIcon)
	ON_COMMAND(ID_VIEW_LARGEICON, OnViewLargeIcon)
	ON_COMMAND(ID_VIEW_LIST, OnViewList)
	ON_UPDATE_COMMAND_UI(ID_VIEW_LIST, OnUpdateViewList)
	ON_COMMAND(ID_VIEW_SMALLICON, OnViewSmallIcon)
	ON_UPDATE_COMMAND_UI(ID_VIEW_SMALLICON, OnUpdateViewSmallIcon)
	ON_NOTIFY_REFLECT(LVN_DELETEITEM, OnDeleteitem)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

CLottoManView::CLottoManView()
{
  //Load up the image lists
  m_ILSmall.Create(IDB_SMALL_IMAGELIST, 16, 1, RGB(255, 0, 255));
  m_ILBig.Create(IDB_BIG_IMAGELIST, 32, 1, RGB(255, 0, 255));

  m_bInPlaceEditing = FALSE;
}

void CLottoManView::OnDraw(CDC* /*pDC*/)
{
  //Nothing much to do in the OnDraw method
	CLottoManDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
}

void CLottoManView::OnInitialUpdate()
{
  //Let the parent do its thing
	CListView::OnInitialUpdate();

  //Turn on full row select (makes the control look more like the VB equivalent)
  SendMessage(LVM_SETEXTENDEDLISTVIEWSTYLE, LVS_EX_FULLROWSELECT, LVS_EX_FULLROWSELECT);

  //Turn on grid lines
  SendMessage(LVM_SETEXTENDEDLISTVIEWSTYLE, LVS_EX_GRIDLINES, LVS_EX_GRIDLINES);

  //remove all the current items
  CListCtrl& rList = GetListCtrl();
  rList.DeleteColumn(1);
  rList.DeleteColumn(0);
  rList.DeleteAllItems();

  //attach the image lists
  rList.SetImageList(&m_ILSmall, LVSIL_SMALL);
  rList.SetImageList(&m_ILBig, LVSIL_NORMAL);

  //create the columns
  rList.InsertColumn(0, _T("Member"), LVCFMT_LEFT, 200);
  rList.InsertColumn(1, _T("Numbers"), LVCFMT_LEFT, 200);

  //Add the wizard
  int nWizIndex = rList.InsertItem(0, _T("Add New Member"), 1);
  rList.SetItemData(nWizIndex, 0);

  //Add the settings item
  CString sCaption;
  sCaption.LoadString(IDS_DRAW_SETTINGS);
  rList.InsertItem(1, sCaption, 3);

  //add the members to the view
  CLottoManDoc* pDoc = GetDocument();
  for (int i=0; i<pDoc->m_Members.GetSize(); i++)
  {
    CMember* pMember = pDoc->m_Members.GetAt(i);
    if (!pMember->m_bMarkedForDeletion)
      AddMemberToView(pDoc->m_nNumbersInDraw, pMember, i);
  }

  //Change the list view style depending on the doc preference
  switch (pDoc->m_nListViewStyle)
  {
    case 0:  OnViewList();      break;
    case 1:  OnViewSmallIcon(); break;
    case 2:  OnViewDetails();   break;
    default: OnViewLargeIcon(); break;
  }
  pDoc->SetModifiedFlag(FALSE);
}

#ifdef _DEBUG
void CLottoManView::AssertValid() const
{
	CListView::AssertValid();
}

void CLottoManView::Dump(CDumpContext& dc) const
{
	CListView::Dump(dc);
}

CLottoManDoc* CLottoManView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CLottoManDoc)));
	return (CLottoManDoc*)m_pDocument;
}
#endif //_DEBUG

void CLottoManView::OnEditSelectall() 
{
	CListCtrl& rList = GetListCtrl();
	for (int i = 0; i < rList.GetItemCount(); i++)
		SetSelected(i, TRUE);
}

void CLottoManView::OnEditInvertselection() 
{
	CListCtrl& rList = GetListCtrl();
	for (int i = 0; i < rList.GetItemCount(); i++)
		SetSelected(i, !IsSelected(i));
}

BOOL CLottoManView::IsSelected(int nIndex) const
{
	return (GetListCtrl().GetItemState(nIndex, LVIS_SELECTED) & LVIS_SELECTED);	
}

BOOL CLottoManView::IsAnythingSelected() const
{
	BOOL bSelected = FALSE;

	CListCtrl& rList = GetListCtrl();
	for (int i = 0; i < rList.GetItemCount(); i++)
  {
		if (IsSelected(i))
    {
      bSelected = TRUE;
      break;
    }
  }

  return bSelected;
}

int CLottoManView::GetCurSel() const 
{
	CListCtrl& rList = GetListCtrl();
	for (int i=0; i<rList.GetItemCount(); i++)
  {
		if (IsSelected(i))
      return i;
  }
	return -1;
}

void CLottoManView::SetSelected(int nIndex, BOOL bSelect) const
{
	CListCtrl& rList = GetListCtrl();
  if (bSelect)
    rList.SetItemState(nIndex, LVIS_SELECTED,	LVIS_SELECTED);
  else
    rList.SetItemState(nIndex, 0,	LVIS_SELECTED);
}

LONG CLottoManView::GetStyle()
{
	LONG lWndStyle = GetWindowLong(GetListCtrl().m_hWnd, GWL_STYLE);
	lWndStyle &= LVS_TYPEMASK;

  return lWndStyle;
}

void CLottoManView::ChangeStyle(LONG nNewStyle)
{
	LONG lWndStyle = GetWindowLong(GetListCtrl().m_hWnd, GWL_STYLE);
	lWndStyle &= ~LVS_TYPEMASK;
	lWndStyle |= nNewStyle |= LVS_EDITLABELS;
	SetWindowLong(GetListCtrl().m_hWnd, GWL_STYLE, lWndStyle);
}

BOOL CLottoManView::IsWizardSelected()
{
  int nCurSel = GetCurSel();
  return (IsItemWizard(nCurSel));
}

BOOL CLottoManView::IsSettingsSelected()
{
  int nCurSel = GetCurSel();
  return (IsItemSettings(nCurSel));
}

void CLottoManView::OnFileDelete() 
{
	CListCtrl& rList = GetListCtrl();
  int nNum = NumberNonWizardSelectedItems();
  BOOL bOkToDelete;
  if (nNum > 1)
  {
    CString sNum;
    sNum.Format(_T("%d"), nNum);
    CString sMsg;
    AfxFormatString1(sMsg, IDP_CONFIRM_MULTIPLE_DELETE, sNum);
    bOkToDelete = (AfxMessageBox(sMsg, MB_YESNO | MB_ICONQUESTION) == IDYES);
  }
  else
  {
    CString sName = rList.GetItemText(GetCurSel(), 0);
    CString sMsg;
    AfxFormatString1(sMsg, IDP_CONFIRM_DELETE, sName);
    bOkToDelete = (AfxMessageBox(sMsg, MB_YESNO | MB_ICONQUESTION) == IDYES);
  }

  if (!bOkToDelete)
    return;

	for (int i = rList.GetItemCount()-1; i >= 1; i--)
  {
		if (IsSelected(i))
    {
      //remove item from configuration by marking it
      //as deleted. When the archive to disk is done upon
      //exiting the app, the object will be skipped
      GetMember(i)->m_bMarkedForDeletion = TRUE;

      // Remove item from the view
      rList.DeleteItem(i);
    }
  }

  GetDocument()->SetModifiedFlag(TRUE);
}

void CLottoManView::OnUpdateFileDelete(CCmdUI* pCmdUI) 
{
  pCmdUI->Enable(IsAnythingSelected() && !IsWizardSelected() && !IsSettingsSelected() && !m_bInPlaceEditing);	
}

void CLottoManView::OnReturn(NMHDR* pNMHDR, LRESULT* pResult) 
{
	OnDblclk(pNMHDR, pResult);
	
	*pResult = 0;
}

void CLottoManView::OnDblclk(NMHDR* /*pNMHDR*/, LRESULT* pResult) 
{
  if (IsWizardSelected() || IsSettingsSelected())
    PostMessage(WM_COMMAND, ID_FILE_OPEN_ITEM);
  else
    PostMessage(WM_COMMAND, ID_FILE_PROPERTIES);
	
	*pResult = 0;
}

void CLottoManView::OnBeginlabeledit(NMHDR* /*pNMHDR*/, LRESULT* pResult) 
{
  //Disallow in place editing of the wizard
	if (IsWizardSelected())
	{
	  *pResult = TRUE;
		return;
	}
  else
  	*pResult = 0;

  m_bInPlaceEditing = TRUE;
}

void CLottoManView::OnEndlabeledit(NMHDR* pNMHDR, LRESULT* pResult) 
{
	LV_DISPINFO* pDispInfo = (LV_DISPINFO*)pNMHDR;
  CListCtrl& rList = GetListCtrl();
  m_bInPlaceEditing = FALSE;

  if (pDispInfo->item.pszText)
  {
    if (_tcslen(pDispInfo->item.pszText) == 0)
    {
      AfxMessageBox(IDP_MISSING_NAME, MB_OK | MB_ICONSTOP);
      PostMessage(WM_COMMAND, ID_FILE_RENAME);
      *pResult = FALSE;
      return;
    }

    CString sName(pDispInfo->item.pszText);
    CLottoManDoc* pDoc = GetDocument();
    if (pDoc->DoesMemberNameExist(sName, GetMemberArrayIndex(pDispInfo->item.iItem)))
    {
      CString sMsg;
      AfxFormatString1(sMsg, IDP_RENAME_UNIQUE_MEMBERNAME, sName);
      AfxMessageBox(sMsg, MB_OK | MB_ICONSTOP);
      PostMessage(WM_COMMAND, ID_FILE_RENAME);
      *pResult = FALSE;
      return;
    }

    CMember* pMember = GetMember(pDispInfo->item.iItem);
    pMember->m_sName = sName;

    //Update on the screen
    TCHAR* pszCaption = pMember->m_sName.GetBuffer(pMember->m_sName.GetLength());
    rList.SetItemText(pDispInfo->item.iItem, 0, pszCaption);
    pMember->m_sName.ReleaseBuffer();

    pDoc->SetModifiedFlag(TRUE);
    *pResult = FALSE;
  }
  else
    *pResult = FALSE;
}

void CLottoManView::OnRclick(NMHDR* /*pNMHDR*/, LRESULT* pResult) 
{
  CPoint point;
  GetCursorPos(&point);
	OnContextMenu(NULL, point);
	*pResult = 0;
}

void CLottoManView::OnContextMenu(CWnd*, CPoint point)
{
	CMenu menu;
  BOOL bUseObjectMenu = (GetListCtrl().GetSelectedCount() == 1);
  if (bUseObjectMenu)
    VERIFY(menu.LoadMenu(IDR_POPUP_VIEW_USER));
  else 
    VERIFY(menu.LoadMenu(IDR_POPUP_VIEW_NOOBJECT));

	CMenu* pPopup = menu.GetSubMenu(0);
	ASSERT(pPopup != NULL);

	CWnd* pWndPopupOwner = this;
	while (pWndPopupOwner->GetStyle() & WS_CHILD)
		pWndPopupOwner = pWndPopupOwner->GetParent();

	pPopup->TrackPopupMenu(TPM_LEFTALIGN | TPM_RIGHTBUTTON, point.x, point.y,
		pWndPopupOwner);
}

void CLottoManView::OnFileOpenItem() 
{
  int nCurSel = GetCurSel();
  if (nCurSel != -1)
  {
    if (IsItemWizard(nCurSel))
    {
      CString sCaption;
      CMember member;
      CLottoManDoc* pDoc = GetDocument();
      member.m_sName = pDoc->CreateUniqueMemberName();

      AfxFormatString1(sCaption, IDS_PROPERTIES_FOR, member.m_sName);
      CMemberPropertySheet dlg(sCaption);
      dlg.m_page1.m_pDoc = pDoc;
      dlg.m_psh.dwFlags |= PSH_NOAPPLYNOW;
      dlg.SetMember(member);
      if (dlg.DoModal() == IDOK)
      {
        //create a new filter
        CMember* pMember = new CMember(dlg.GetMember());

        //add it to the configuration
        pDoc->m_Members.Add(pMember);

        //Add it to the view
        AddMemberToView(pDoc->m_nNumbersInDraw, pMember, pDoc->m_Members.GetSize()-1);

        //Update the modified flag
        pDoc->SetModifiedFlag(TRUE);

        //Unselect the wizard
        SetSelected(0, FALSE);

        //Select the new item
        SetSelected(GetListCtrl().GetItemCount()-1, TRUE);
      }
    }
    else if (IsItemSettings(nCurSel))
    {
      CLottoManDoc* pDoc = GetDocument();

      CSettingsPropertySheet dlg(IDS_DRAW_SETTINGS);
      dlg.m_psh.dwFlags |= PSH_NOAPPLYNOW;
      dlg.m_page1.m_bBonusNumberPicked = pDoc->m_bBonusNumberPicked;
      dlg.m_page1.m_nNumbersInDraw = pDoc->m_nNumbersInDraw;
      dlg.m_page1.m_nMaxLottoNumber = pDoc->m_nMaxLottoNumber;
      if (dlg.DoModal() == IDOK)
      {
        //Save the new settings in the document
        pDoc->m_bBonusNumberPicked = dlg.m_page1.m_bBonusNumberPicked;
        pDoc->m_nNumbersInDraw     = dlg.m_page1.m_nNumbersInDraw;
        pDoc->m_nMaxLottoNumber    = dlg.m_page1.m_nMaxLottoNumber;

        //Update the modified flag  
        pDoc->SetModifiedFlag(TRUE);

        //Also force a refresh of all current members
        CListCtrl& rList = GetListCtrl();
        for (int i=2; i<rList.GetItemCount(); i++)
        {
          CMember* pMember = GetMember(i);
          UpdateMemberToView(pDoc->m_nNumbersInDraw, pMember, i);
        }
      }
    }
  }
}

void CLottoManView::OnUpdateFileOpenItem(CCmdUI* pCmdUI) 
{
	CListCtrl& rList = GetListCtrl();
  pCmdUI->Enable(rList.GetSelectedCount() == 1 && (IsWizardSelected() || IsSettingsSelected()));			
}

void CLottoManView::OnFileRename() 
{
  CListCtrl& rList = GetListCtrl();
  rList.EditLabel(GetCurSel());
}

void CLottoManView::OnUpdateFileRename(CCmdUI* pCmdUI) 
{
	CListCtrl& rList = GetListCtrl();
  pCmdUI->Enable(rList.GetSelectedCount() == 1 && !IsWizardSelected() && !IsSettingsSelected());	
}

BOOL CLottoManView::PreTranslateMessage(MSG* pMsg) 
{
	if ((((pMsg->message == WM_KEYDOWN || pMsg->message == WM_SYSKEYDOWN) && // If we hit a key and
		(pMsg->wParam == VK_F10) && (GetKeyState(VK_SHIFT) & ~1)) != 0) ||	// it's Shift+F10 OR
		(pMsg->message == WM_CONTEXTMENU))									// Natural keyboard key
	{
		CRect rect;
		GetClientRect(rect);
		ClientToScreen(rect);

		CPoint point = rect.TopLeft();
		point.Offset(5, 5);
		OnContextMenu(NULL, point);

		return TRUE;
	}
  else if (pMsg->message == WM_KEYDOWN && pMsg->wParam == VK_DELETE && !m_bInPlaceEditing)
  {
    PostMessage(WM_COMMAND, ID_FILE_DELETE);
    return TRUE;
  }

	return CListView::PreTranslateMessage(pMsg);
}

void CLottoManView::OnFileProperties() 
{
  if (GetListCtrl().GetSelectedCount() != 1 || IsSelected(0))
    return;

  int nCurSel = GetCurSel();

  CLottoManDoc* pDoc = GetDocument();
  CString sCaption;
  CMember* pMember = GetMember(nCurSel);
  AfxFormatString1(sCaption, IDS_PROPERTIES_FOR, pMember->m_sName);
  CMemberPropertySheet dlg(sCaption);
  dlg.m_psh.dwFlags |= PSH_NOAPPLYNOW;
  dlg.SetMember(*pMember);
  dlg.SetMemberArrayIndex(GetMemberArrayIndex(nCurSel));
  dlg.m_page1.m_pDoc = pDoc;
  if (dlg.DoModal() == IDOK)
  {
    *pMember = dlg.GetMember();

    //Update on the screen
    UpdateMemberToView(pDoc->m_nNumbersInDraw, pMember, nCurSel);

    pDoc->SetModifiedFlag(TRUE);
  }
}

int CLottoManView::GetMemberArrayIndex(int nIndex)
{
  ASSERT(!IsItemWizard(nIndex));
  ASSERT(!IsItemSettings(nIndex));
	LottoViewItem* pItem = (LottoViewItem*) GetListCtrl().GetItemData(nIndex);
	return pItem->nData;
}

CMember* CLottoManView::GetMember(int nIndex)
{
  ASSERT(!IsItemWizard(nIndex));
  ASSERT(!IsItemSettings(nIndex));
	LottoViewItem* pItem = (LottoViewItem*) GetListCtrl().GetItemData(nIndex);
	return (CMember*) pItem->pData;
}

void CLottoManView::OnUpdateFileProperties(CCmdUI* pCmdUI) 
{
	CListCtrl& rList = GetListCtrl();
  pCmdUI->Enable(rList.GetSelectedCount() == 1 && NumberNonWizardSelectedItems() == 1);	
}

int CLottoManView::GetImageItemData(int nIndex) const
{
  LV_ITEM lvItem;
  memset(&lvItem, 0, sizeof(LV_ITEM));
  lvItem.mask = LVIF_IMAGE;
  lvItem.iItem = nIndex;
  GetListCtrl().GetItem(&lvItem);
  return lvItem.iImage;
}

BOOL CLottoManView::IsItemWizard(int nIndex) const
{
  return GetImageItemData(nIndex) == 1;
}

BOOL CLottoManView::IsItemSettings(int nIndex) const
{
  return GetImageItemData(nIndex) == 3;
}

int CLottoManView::NumberNonWizardSelectedItems() const
{
  int nCount = 0;
	CListCtrl& rList = GetListCtrl();
	for (int i = 0; i < rList.GetItemCount(); i++)
  {
    if (IsSelected(i) && !IsItemWizard(i) && !IsItemSettings(i))
      ++nCount;
  }
  return nCount;
}

void CLottoManView::ShowRememberedColumnWidth(const CString& sEntry)
{
  CWinApp* pApp = AfxGetApp();
  int nWidth =  pApp->GetProfileInt(_T("General"), sEntry, 0);
  if (nWidth)
    GetListCtrl().SetColumnWidth(0, nWidth);
}

void CLottoManView::StoreColumnWidth(const CString& sEntry)
{
  CWinApp* pApp = AfxGetApp();
  int nWidth =  GetListCtrl().GetColumnWidth(0);
  pApp->WriteProfileInt(_T("General"), sEntry, nWidth);
}

void CLottoManView::OnFileCheckResults() 
{
  CResultsDlg dlg;
  dlg.m_pDoc = GetDocument();
  dlg.DoModal();
}

void CLottoManView::AddMemberToView(int nNumbersInDraw, CMember* pMember, int nMemberIndex)
{
  CListCtrl& rList = GetListCtrl();
  TCHAR* pszCaption = pMember->m_sName.GetBuffer(pMember->m_sName.GetLength());
  LV_ITEM lvItem;
  memset(&lvItem, 0, sizeof(LV_ITEM));
  lvItem.mask = LVIF_TEXT | LVIF_IMAGE | LVIF_PARAM;
  lvItem.iItem = rList.GetItemCount();
  lvItem.pszText = pszCaption;

  if (pMember->m_bEnabled)
    lvItem.iImage = 0;
  else
    lvItem.iImage = 2;
  
	LottoViewItem* pItem = new LottoViewItem;
	pItem->pData = pMember;
	pItem->nData = nMemberIndex;
  lvItem.lParam = (LPARAM) pItem;

  int nIndex = rList.InsertItem(&lvItem);
  pMember->m_sName.ReleaseBuffer();

  CString sNumbers;
  for (int i=0; i<nNumbersInDraw; i++)
  {
    CString sTemp;
    sTemp.Format(_T("%d"), pMember->m_Numbers.GetAt(i));
    sNumbers += sTemp;
    if (i < (nNumbersInDraw - 1))
      sNumbers += _T(", ");
  }

  rList.SetItemText(nIndex, 1, sNumbers);
}

void CLottoManView::UpdateMemberToView(int nNumbersInDraw, CMember* pMember, int nIndex)
{
  CListCtrl& rList = GetListCtrl();

  //Update their name
  rList.SetItemText(nIndex, 0, pMember->m_sName);

  //Also if necessary update their icon
  LV_ITEM lvItem;
  lvItem.mask = LVIF_TEXT | LVIF_IMAGE;
  lvItem.pszText = (LPTSTR) (LPCTSTR) pMember->m_sName;
  lvItem.iItem = nIndex;
  lvItem.iSubItem = 0;
  if (pMember->m_bEnabled)
    lvItem.iImage = 0;
  else
    lvItem.iImage = 2;
  rList.SetItem(&lvItem);

  //Update the numbers they have
  CString sNumbers;
  for (int i=0; i<nNumbersInDraw; i++)
  {
    CString sTemp;
    sTemp.Format(_T("%d"), pMember->m_Numbers.GetAt(i));
    sNumbers += sTemp;
    if (i < (nNumbersInDraw - 1))
      sNumbers += _T(", ");
  }
  rList.SetItemText(nIndex, 1, sNumbers);
}

void CLottoManView::OnUpdateFileCheckResults(CCmdUI* pCmdUI) 
{
  pCmdUI->Enable(GetDocument()->NumberOfMembers());
}

void CLottoManView::OnViewDetails() 
{
  ChangeStyle(LVS_REPORT);
  GetDocument()->m_nListViewStyle = 2;
  GetDocument()->SetModifiedFlag(TRUE);
}

void CLottoManView::OnUpdateViewDetails(CCmdUI* pCmdUI) 
{
	LONG nStyle = GetWindowLong(GetSafeHwnd(), GWL_STYLE);
  pCmdUI->SetCheck((nStyle & LVS_TYPEMASK) == LVS_REPORT);
}

void CLottoManView::OnViewLargeIcon() 
{
  ChangeStyle(LVS_ICON);
  GetDocument()->m_nListViewStyle = 3;
  GetDocument()->SetModifiedFlag(TRUE);
}

void CLottoManView::OnUpdateViewLargeIcon(CCmdUI* pCmdUI) 
{
	LONG nStyle = GetWindowLong(GetSafeHwnd(), GWL_STYLE);
  pCmdUI->SetCheck((nStyle & LVS_TYPEMASK) == LVS_ICON);
}

void CLottoManView::OnViewList() 
{
  ChangeStyle(LVS_LIST);
  GetDocument()->m_nListViewStyle = 0;
  GetDocument()->SetModifiedFlag(TRUE);
}

void CLottoManView::OnUpdateViewList(CCmdUI* pCmdUI) 
{
	LONG nStyle = GetWindowLong(GetSafeHwnd(), GWL_STYLE);
  pCmdUI->SetCheck((nStyle & LVS_TYPEMASK) == LVS_LIST);
}

void CLottoManView::OnViewSmallIcon() 
{
  ChangeStyle(LVS_SMALLICON);
  GetDocument()->m_nListViewStyle = 1;
  GetDocument()->SetModifiedFlag(TRUE);
}

void CLottoManView::OnUpdateViewSmallIcon(CCmdUI* pCmdUI) 
{
	LONG nStyle = GetWindowLong(GetSafeHwnd(), GWL_STYLE);
  pCmdUI->SetCheck((nStyle & LVS_TYPEMASK) == LVS_SMALLICON);
}

void CLottoManView::OnDeleteitem(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_LISTVIEW* pNMListView = (NM_LISTVIEW*)pNMHDR;
	
  //Delete the memory associated with each item
	LottoViewItem* pItem = (LottoViewItem*) GetListCtrl().GetItemData(pNMListView->iItem);
	if (pItem)
		delete pItem;
	
	*pResult = 0;
}

